@@ -14,37 +14,84 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
-# pylint: disable=C,R,W
 import json
-from typing import Any, List
+from enum import Enum
+from typing import Any, Dict, Hashable, List, Optional, Type, Union
 
+from flask_appbuilder.security.sqla.models import User
 from sqlalchemy import and_, Boolean, Column, Integer, String, Text
 from sqlalchemy.ext.declarative import declared_attr
-from sqlalchemy.orm import foreign, relationship
+from sqlalchemy.orm import foreign, Query, relationship, RelationshipProperty
 
-from superset.models.core import Slice
-from superset.models.helpers import AuditMixinNullable, ImportMixin
+from superset import security_manager
+from superset.constants import NULL_STRING
+from superset.models.helpers import AuditMixinNullable, ImportMixin, QueryResult
+from superset.models.slice import Slice
+from superset.typing import FilterValue, FilterValues, QueryObjectDict
 from superset.utils import core as utils
 
+METRIC_FORM_DATA_PARAMS = [
+    "metric",
+    "metrics",
+    "metric_2",
+    "percent_metrics",
+    "secondary_metric",
+    "size",
+    "timeseries_limit_metric",
+    "x",
+    "y",
+]
 
-class BaseDatasource(AuditMixinNullable, ImportMixin):
+COLUMN_FORM_DATA_PARAMS = [
+    "all_columns",
+    "all_columns_x",
+    "columns",
+    "entity",
+    "groupby",
+    "order_by_cols",
+    "series",
+]
+
+
+class DatasourceKind(str, Enum):
+    VIRTUAL = "virtual"
+    PHYSICAL = "physical"
+
+
+class BaseDatasource(
+    AuditMixinNullable, ImportMixin
+):  # pylint: disable=too-many-public-methods
     """A common interface to objects that are queryable
     (tables and datasources)"""
 
     # ---------------------------------------------------------------
     # class attributes to define when deriving BaseDatasource
     # ---------------------------------------------------------------
-    __tablename__ = None  # {connector_name}_datasource
-    type = None  # datasoure type, str to be defined when deriving this class
-    baselink = None  # url portion pointing to ModelView endpoint
-    column_class = None  # link to derivative of BaseColumn
-    metric_class = None  # link to derivative of BaseMetric
-    owner_class = None
+    __tablename__: Optional[str] = None  # {connector_name}_datasource
+    baselink: Optional[str] = None  # url portion pointing to ModelView endpoint
+
+    @property
+    def column_class(self) -> Type["BaseColumn"]:
+        # link to derivative of BaseColumn
+        raise NotImplementedError()
+
+    @property
+    def metric_class(self) -> Type["BaseMetric"]:
+        # link to derivative of BaseMetric
+        raise NotImplementedError()
+
+    owner_class: Optional[User] = None
 
     # Used to do code highlighting when displaying the query in the UI
-    query_language = None
+    query_language: Optional[str] = None
 
-    name = None  # can be a Column or a property pointing to one
+    # Only some datasources support Row Level Security
+    is_rls_supported: bool = False
+
+    @property
+    def name(self) -> str:
+        # can be a Column or a property pointing to one
+        raise NotImplementedError()
 
     # ---------------------------------------------------------------
 
@@ -58,13 +105,21 @@ class BaseDatasource(AuditMixinNullable, ImportMixin):
     cache_timeout = Column(Integer)
     params = Column(String(1000))
     perm = Column(String(1000))
+    schema_perm = Column(String(1000))
 
-    sql = None
-    owners = None
-    update_from_object_fields = None
+    sql: Optional[str] = None
+    owners: List[User]
+    update_from_object_fields: List[str]
+
+    @property
+    def kind(self) -> str:
+        if self.sql:
+            return DatasourceKind.VIRTUAL.value
+
+        return DatasourceKind.PHYSICAL.value
 
     @declared_attr
-    def slices(self):
+    def slices(self) -> RelationshipProperty:
         return relationship(
             "Slice",
             primaryjoin=lambda: and_(
@@ -79,68 +134,71 @@ class BaseDatasource(AuditMixinNullable, ImportMixin):
     metrics: List[Any] = []
 
     @property
-    def uid(self):
+    def type(self) -> str:
+        raise NotImplementedError()
+
+    @property
+    def uid(self) -> str:
         """Unique id across datasource types"""
         return f"{self.id}__{self.type}"
 
     @property
-    def column_names(self):
+    def column_names(self) -> List[str]:
         return sorted([c.column_name for c in self.columns], key=lambda x: x or "")
 
     @property
-    def columns_types(self):
+    def columns_types(self) -> Dict[str, str]:
         return {c.column_name: c.type for c in self.columns}
 
     @property
-    def main_dttm_col(self):
+    def main_dttm_col(self) -> str:
         return "timestamp"
 
     @property
-    def datasource_name(self):
+    def datasource_name(self) -> str:
         raise NotImplementedError()
 
     @property
-    def connection(self):
+    def connection(self) -> Optional[str]:
         """String representing the context of the Datasource"""
         return None
 
     @property
-    def schema(self):
+    def schema(self) -> Optional[str]:
         """String representing the schema of the Datasource (if it applies)"""
         return None
 
     @property
-    def filterable_column_names(self):
+    def filterable_column_names(self) -> List[str]:
         return sorted([c.column_name for c in self.columns if c.filterable])
 
     @property
-    def dttm_cols(self):
+    def dttm_cols(self) -> List[str]:
         return []
 
     @property
-    def url(self):
+    def url(self) -> str:
         return "/{}/edit/{}".format(self.baselink, self.id)
 
     @property
-    def explore_url(self):
+    def explore_url(self) -> str:
         if self.default_endpoint:
             return self.default_endpoint
-        else:
-            return "/superset/explore/{obj.type}/{obj.id}/".format(obj=self)
+        return f"/superset/explore/{self.type}/{self.id}/"
 
     @property
-    def column_formats(self):
+    def column_formats(self) -> Dict[str, Optional[str]]:
         return {m.metric_name: m.d3format for m in self.metrics if m.d3format}
 
-    def add_missing_metrics(self, metrics):
-        exisiting_metrics = {m.metric_name for m in self.metrics}
+    def add_missing_metrics(self, metrics: List["BaseMetric"]) -> None:
+        existing_metrics = {m.metric_name for m in self.metrics}
         for metric in metrics:
-            if metric.metric_name not in exisiting_metrics:
+            if metric.metric_name not in existing_metrics:
                 metric.table_id = self.id
-                self.metrics += [metric]
+                self.metrics.append(metric)
 
     @property
-    def short_data(self):
+    def short_data(self) -> Dict[str, Any]:
         """Data representation of the datasource sent to the frontend"""
         return {
             "edit_url": self.url,
@@ -154,18 +212,22 @@ class BaseDatasource(AuditMixinNullable, ImportMixin):
         }
 
     @property
-    def select_star(self):
+    def select_star(self) -> Optional[str]:
         pass
 
     @property
-    def data(self):
+    def data(self) -> Dict[str, Any]:
         """Data representation of the datasource sent to the frontend"""
         order_by_choices = []
         # self.column_names return sorted column_names
-        for s in self.column_names:
-            s = str(s or "")
-            order_by_choices.append((json.dumps([s, True]), s + " [asc]"))
-            order_by_choices.append((json.dumps([s, False]), s + " [desc]"))
+        for column_name in self.column_names:
+            column_name = str(column_name or "")
+            order_by_choices.append(
+                (json.dumps([column_name, True]), column_name + " [asc]")
+            )
+            order_by_choices.append(
+                (json.dumps([column_name, False]), column_name + " [desc]")
+            )
 
         verbose_map = {"__timestamp": "Time"}
         verbose_map.update(
@@ -204,49 +266,118 @@ class BaseDatasource(AuditMixinNullable, ImportMixin):
             "select_star": self.select_star,
         }
 
+    def data_for_slices(self, slices: List[Slice]) -> Dict[str, Any]:
+        """
+        The representation of the datasource containing only the required data
+        to render the provided slices.
+
+        Used to reduce the payload when loading a dashboard.
+        """
+        data = self.data
+        metric_names = set()
+        column_names = set()
+        for slc in slices:
+            form_data = slc.form_data
+
+            # pull out all required metrics from the form_data
+            for param in METRIC_FORM_DATA_PARAMS:
+                for metric in utils.get_iterable(form_data.get(param) or []):
+                    metric_names.add(utils.get_metric_name(metric))
+
+                    if utils.is_adhoc_metric(metric):
+                        column_names.add(
+                            (metric.get("column") or {}).get("column_name")
+                        )
+
+            # pull out all required columns from the form_data
+            for filter_ in form_data.get("adhoc_filters") or []:
+                if filter_["clause"] == "WHERE" and filter_.get("subject"):
+                    column_names.add(filter_.get("subject"))
+
+            for param in COLUMN_FORM_DATA_PARAMS:
+                for column in utils.get_iterable(form_data.get(param) or []):
+                    column_names.add(column)
+
+        filtered_metrics = [
+            metric
+            for metric in data["metrics"]
+            if metric["metric_name"] in metric_names
+        ]
+
+        filtered_columns = [
+            column
+            for column in data["columns"]
+            if column["column_name"] in column_names
+        ]
+
+        del data["description"]
+        data.update({"metrics": filtered_metrics})
+        data.update({"columns": filtered_columns})
+        verbose_map = {"__timestamp": "Time"}
+        verbose_map.update(
+            {
+                metric["metric_name"]: metric["verbose_name"] or metric["metric_name"]
+                for metric in filtered_metrics
+            }
+        )
+        verbose_map.update(
+            {
+                column["column_name"]: column["verbose_name"] or column["column_name"]
+                for column in filtered_columns
+            }
+        )
+        data["verbose_map"] = verbose_map
+
+        return data
+
     @staticmethod
     def filter_values_handler(
-        values, target_column_is_numeric=False, is_list_target=False
-    ):
-        def handle_single_value(v):
+        values: Optional[FilterValues],
+        target_column_is_numeric: bool = False,
+        is_list_target: bool = False,
+    ) -> Optional[FilterValues]:
+        if values is None:
+            return None
+
+        def handle_single_value(value: Optional[FilterValue]) -> Optional[FilterValue]:
             # backward compatibility with previous <select> components
-            if isinstance(v, str):
-                v = v.strip("\t\n'\"")
+            if isinstance(value, str):
+                value = value.strip("\t\n'\"")
                 if target_column_is_numeric:
                     # For backwards compatibility and edge cases
                     # where a column data type might have changed
-                    v = utils.string_to_num(v)
-                if v == "<NULL>":
+                    value = utils.cast_to_num(value)
+                if value == NULL_STRING:
                     return None
-                elif v == "<empty string>":
+                if value == "<empty string>":
                     return ""
-            return v
+            return value
 
         if isinstance(values, (list, tuple)):
-            values = [handle_single_value(v) for v in values]
+            values = [handle_single_value(v) for v in values]  # type: ignore
         else:
             values = handle_single_value(values)
         if is_list_target and not isinstance(values, (tuple, list)):
-            values = [values]
+            values = [values]  # type: ignore
         elif not is_list_target and isinstance(values, (tuple, list)):
-            if len(values) > 0:
+            if values:
                 values = values[0]
             else:
                 values = None
         return values
 
-    def external_metadata(self):
+    def external_metadata(self) -> List[Dict[str, str]]:
         """Returns column information from the external system"""
         raise NotImplementedError()
 
-    def get_query_str(self, query_obj):
+    def get_query_str(self, query_obj: QueryObjectDict) -> str:
         """Returns a query as a string
 
         This is used to be displayed to the user so that she/he can
         understand what is taking place behind the scene"""
         raise NotImplementedError()
 
-    def query(self, query_obj):
+    def query(self, query_obj: QueryObjectDict) -> QueryResult:
         """Executes the query and returns a dataframe
 
         query_obj is a dictionary representing Superset's query interface.
@@ -254,7 +385,7 @@ class BaseDatasource(AuditMixinNullable, ImportMixin):
         """
         raise NotImplementedError()
 
-    def values_for_column(self, column_name, limit=10000):
+    def values_for_column(self, column_name: str, limit: int = 10000) -> List[Any]:
         """Given a column, returns an iterable of distinct values
 
         This is used to populate the dropdown showing a list of
@@ -262,30 +393,39 @@ class BaseDatasource(AuditMixinNullable, ImportMixin):
         raise NotImplementedError()
 
     @staticmethod
-    def default_query(qry):
+    def default_query(qry: Query) -> Query:
         return qry
 
-    def get_column(self, column_name):
+    def get_column(self, column_name: Optional[str]) -> Optional["BaseColumn"]:
+        if not column_name:
+            return None
         for col in self.columns:
             if col.column_name == column_name:
                 return col
+        return None
 
-    def get_fk_many_from_list(self, object_list, fkmany, fkmany_class, key_attr):
+    @staticmethod
+    def get_fk_many_from_list(
+        object_list: List[Any],
+        fkmany: List[Column],
+        fkmany_class: Type[Union["BaseColumn", "BaseMetric"]],
+        key_attr: str,
+    ) -> List[Column]:  # pylint: disable=too-many-locals
         """Update ORM one-to-many list from object list
 
         Used for syncing metrics and columns using the same code"""
 
         object_dict = {o.get(key_attr): o for o in object_list}
-        object_keys = [o.get(key_attr) for o in object_list]
 
         # delete fks that have been removed
-        fkmany = [o for o in fkmany if getattr(o, key_attr) in object_keys]
+        fkmany = [o for o in fkmany if getattr(o, key_attr) in object_dict]
 
         # sync existing fks
         for fk in fkmany:
             obj = object_dict.get(getattr(fk, key_attr))
-            for attr in fkmany_class.update_from_object_fields:
-                setattr(fk, attr, obj.get(attr))
+            if obj:
+                for attr in fkmany_class.update_from_object_fields:
+                    setattr(fk, attr, obj.get(attr))
 
         # create new fks
         new_fks = []
@@ -303,7 +443,7 @@ class BaseDatasource(AuditMixinNullable, ImportMixin):
         fkmany += new_fks
         return fkmany
 
-    def update_from_object(self, obj):
+    def update_from_object(self, obj: Dict[str, Any]) -> None:
         """Update datasource from a data structure
 
         The UI's table editor crafts a complex data structure that
@@ -320,27 +460,57 @@ class BaseDatasource(AuditMixinNullable, ImportMixin):
         self.owners = obj.get("owners", [])
 
         # Syncing metrics
-        metrics = self.get_fk_many_from_list(
-            obj.get("metrics"), self.metrics, self.metric_class, "metric_name"
+        metrics = (
+            self.get_fk_many_from_list(
+                obj["metrics"], self.metrics, self.metric_class, "metric_name"
+            )
+            if self.metric_class and "metrics" in obj
+            else []
         )
         self.metrics = metrics
 
         # Syncing columns
-        self.columns = self.get_fk_many_from_list(
-            obj.get("columns"), self.columns, self.column_class, "column_name"
+        self.columns = (
+            self.get_fk_many_from_list(
+                obj["columns"], self.columns, self.column_class, "column_name"
+            )
+            if self.column_class and "columns" in obj
+            else []
         )
 
-    def get_extra_cache_keys(self, query_obj) -> List[Any]:
+    def get_extra_cache_keys(  # pylint: disable=no-self-use
+        self, query_obj: QueryObjectDict  # pylint: disable=unused-argument
+    ) -> List[Hashable]:
         """ If a datasource needs to provide additional keys for calculation of
         cache keys, those can be provided via this method
+
+        :param query_obj: The dict representation of a query object
+        :return: list of keys
         """
         return []
 
+    def __hash__(self) -> int:
+        return hash(self.uid)
+
+    def __eq__(self, other: object) -> bool:
+        if not isinstance(other, BaseDatasource):
+            return NotImplemented
+        return self.uid == other.uid
+
+    def raise_for_access(self) -> None:
+        """
+        Raise an exception if the user cannot access the resource.
+
+        :raises SupersetSecurityException: If the user cannot access the resource
+        """
+
+        security_manager.raise_for_access(datasource=self)
+
 
 class BaseColumn(AuditMixinNullable, ImportMixin):
     """Interface for column"""
 
-    __tablename__ = None  # {connector_name}_column
+    __tablename__: Optional[str] = None  # {connector_name}_column
 
     id = Column(Integer, primary_key=True)
     column_name = Column(String(255), nullable=False)
@@ -355,8 +525,8 @@ class BaseColumn(AuditMixinNullable, ImportMixin):
     # [optional] Set this to support import/export functionality
     export_fields: List[Any] = []
 
-    def __repr__(self):
-        return self.column_name
+    def __repr__(self) -> str:
+        return str(self.column_name)
 
     num_types = (
         "DOUBLE",
@@ -370,27 +540,31 @@ class BaseColumn(AuditMixinNullable, ImportMixin):
         "DECIMAL",
         "MONEY",
     )
-    date_types = ("DATE", "TIME", "DATETIME")
+    date_types = ("DATE", "TIME")
     str_types = ("VARCHAR", "STRING", "CHAR")
 
     @property
-    def is_num(self):
-        return self.type and any([t in self.type.upper() for t in self.num_types])
+    def is_numeric(self) -> bool:
+        return self.type and any(map(lambda t: t in self.type.upper(), self.num_types))
 
     @property
-    def is_time(self):
-        return self.type and any([t in self.type.upper() for t in self.date_types])
+    def is_temporal(self) -> bool:
+        return self.type and any(map(lambda t: t in self.type.upper(), self.date_types))
 
     @property
-    def is_string(self):
-        return self.type and any([t in self.type.upper() for t in self.str_types])
+    def is_string(self) -> bool:
+        return self.type and any(map(lambda t: t in self.type.upper(), self.str_types))
 
     @property
-    def expression(self):
+    def expression(self) -> Column:
         raise NotImplementedError()
 
     @property
-    def data(self):
+    def python_date_format(self) -> Column:
+        raise NotImplementedError()
+
+    @property
+    def data(self) -> Dict[str, Any]:
         attrs = (
             "id",
             "column_name",
@@ -401,7 +575,6 @@ class BaseColumn(AuditMixinNullable, ImportMixin):
             "groupby",
             "is_dttm",
             "type",
-            "python_date_format",
         )
         return {s: getattr(self, s) for s in attrs if hasattr(self, s)}
 
@@ -410,14 +583,13 @@ class BaseMetric(AuditMixinNullable, ImportMixin):
 
     """Interface for Metrics"""
 
-    __tablename__ = None  # {connector_name}_metric
+    __tablename__: Optional[str] = None  # {connector_name}_metric
 
     id = Column(Integer, primary_key=True)
     metric_name = Column(String(255), nullable=False)
     verbose_name = Column(String(1024))
     metric_type = Column(String(32))
     description = Column(Text)
-    is_restricted = Column(Boolean, default=False, nullable=True)
     d3format = Column(String(128))
     warning_text = Column(Text)
 
@@ -436,15 +608,15 @@ class BaseMetric(AuditMixinNullable, ImportMixin):
     """
 
     @property
-    def perm(self):
+    def perm(self) -> Optional[str]:
         raise NotImplementedError()
 
     @property
-    def expression(self):
+    def expression(self) -> Column:
         raise NotImplementedError()
 
     @property
-    def data(self):
+    def data(self) -> Dict[str, Any]:
         attrs = (
             "id",
             "metric_name",
