@@ -14,39 +14,32 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
+# isort:skip_file
 """Unit tests for Superset"""
 import json
 import unittest
+from random import random
 
-from flask import escape
+from flask import escape, url_for
 from sqlalchemy import func
 
+import tests.test_app
 from superset import db, security_manager
 from superset.connectors.sqla.models import SqlaTable
 from superset.models import core as models
+from superset.models.dashboard import Dashboard
+from superset.models.slice import Slice
+
 from .base_tests import SupersetTestCase
 
 
-class DashboardTests(SupersetTestCase):
-    def __init__(self, *args, **kwargs):
-        super(DashboardTests, self).__init__(*args, **kwargs)
-
-    @classmethod
-    def setUpClass(cls):
-        pass
-
-    def setUp(self):
-        pass
-
-    def tearDown(self):
-        pass
-
+class TestDashboard(SupersetTestCase):
     def get_mock_positions(self, dash):
         positions = {"DASHBOARD_VERSION_KEY": "v2"}
         for i, slc in enumerate(dash.slices):
             id = "DASHBOARD_CHART_TYPE-{}".format(i)
             d = {
-                "type": "DASHBOARD_CHART_TYPE",
+                "type": "CHART",
                 "id": id,
                 "children": [],
                 "meta": {"width": 4, "height": 50, "chartId": slc.id},
@@ -57,23 +50,26 @@ class DashboardTests(SupersetTestCase):
     def test_dashboard(self):
         self.login(username="admin")
         urls = {}
-        for dash in db.session.query(models.Dashboard).all():
+        for dash in db.session.query(Dashboard).all():
             urls[dash.dashboard_title] = dash.url
         for title, url in urls.items():
             assert escape(title) in self.client.get(url).data.decode("utf-8")
 
+    def test_superset_dashboard_url(self):
+        url_for("Superset.dashboard", dashboard_id_or_slug=1)
+
     def test_new_dashboard(self):
         self.login(username="admin")
-        dash_count_before = db.session.query(func.count(models.Dashboard.id)).first()[0]
+        dash_count_before = db.session.query(func.count(Dashboard.id)).first()[0]
         url = "/dashboard/new/"
         resp = self.get_resp(url)
         self.assertIn("[ untitled dashboard ]", resp)
-        dash_count_after = db.session.query(func.count(models.Dashboard.id)).first()[0]
-        self.assertEquals(dash_count_before + 1, dash_count_after)
+        dash_count_after = db.session.query(func.count(Dashboard.id)).first()[0]
+        self.assertEqual(dash_count_before + 1, dash_count_after)
 
     def test_dashboard_modes(self):
         self.login(username="admin")
-        dash = db.session.query(models.Dashboard).filter_by(slug="births").first()
+        dash = db.session.query(Dashboard).filter_by(slug="births").first()
         url = dash.url
         if dash.url.find("?") == -1:
             url += "?"
@@ -86,7 +82,7 @@ class DashboardTests(SupersetTestCase):
 
     def test_save_dash(self, username="admin"):
         self.login(username=username)
-        dash = db.session.query(models.Dashboard).filter_by(slug="births").first()
+        dash = db.session.query(Dashboard).filter_by(slug="births").first()
         positions = self.get_mock_positions(dash)
         data = {
             "css": "",
@@ -100,7 +96,7 @@ class DashboardTests(SupersetTestCase):
 
     def test_save_dash_with_filter(self, username="admin"):
         self.login(username=username)
-        dash = db.session.query(models.Dashboard).filter_by(slug="world_health").first()
+        dash = db.session.query(Dashboard).filter_by(slug="world_health").first()
 
         positions = self.get_mock_positions(dash)
         filters = {str(dash.slices[0].id): {"region": ["North America"]}}
@@ -117,9 +113,7 @@ class DashboardTests(SupersetTestCase):
         resp = self.get_resp(url, data=dict(data=json.dumps(data)))
         self.assertIn("SUCCESS", resp)
 
-        updatedDash = (
-            db.session.query(models.Dashboard).filter_by(slug="world_health").first()
-        )
+        updatedDash = db.session.query(Dashboard).filter_by(slug="world_health").first()
         new_url = updatedDash.url
         self.assertIn("region", new_url)
 
@@ -128,7 +122,7 @@ class DashboardTests(SupersetTestCase):
 
     def test_save_dash_with_invalid_filters(self, username="admin"):
         self.login(username=username)
-        dash = db.session.query(models.Dashboard).filter_by(slug="world_health").first()
+        dash = db.session.query(Dashboard).filter_by(slug="world_health").first()
 
         # add an invalid filter slice
         positions = self.get_mock_positions(dash)
@@ -146,15 +140,13 @@ class DashboardTests(SupersetTestCase):
         resp = self.get_resp(url, data=dict(data=json.dumps(data)))
         self.assertIn("SUCCESS", resp)
 
-        updatedDash = (
-            db.session.query(models.Dashboard).filter_by(slug="world_health").first()
-        )
+        updatedDash = db.session.query(Dashboard).filter_by(slug="world_health").first()
         new_url = updatedDash.url
         self.assertNotIn("region", new_url)
 
     def test_save_dash_with_dashboard_title(self, username="admin"):
         self.login(username=username)
-        dash = db.session.query(models.Dashboard).filter_by(slug="births").first()
+        dash = db.session.query(Dashboard).filter_by(slug="births").first()
         origin_title = dash.dashboard_title
         positions = self.get_mock_positions(dash)
         data = {
@@ -165,9 +157,7 @@ class DashboardTests(SupersetTestCase):
         }
         url = "/superset/save_dash/{}/".format(dash.id)
         self.get_resp(url, data=dict(data=json.dumps(data)))
-        updatedDash = (
-            db.session.query(models.Dashboard).filter_by(slug="births").first()
-        )
+        updatedDash = db.session.query(Dashboard).filter_by(slug="births").first()
         self.assertEqual(updatedDash.dashboard_title, "new title")
         # bring back dashboard original title
         data["dashboard_title"] = origin_title
@@ -175,7 +165,7 @@ class DashboardTests(SupersetTestCase):
 
     def test_save_dash_with_colors(self, username="admin"):
         self.login(username=username)
-        dash = db.session.query(models.Dashboard).filter_by(slug="births").first()
+        dash = db.session.query(Dashboard).filter_by(slug="births").first()
         positions = self.get_mock_positions(dash)
         new_label_colors = {"data value": "random color"}
         data = {
@@ -189,9 +179,7 @@ class DashboardTests(SupersetTestCase):
         }
         url = "/superset/save_dash/{}/".format(dash.id)
         self.get_resp(url, data=dict(data=json.dumps(data)))
-        updatedDash = (
-            db.session.query(models.Dashboard).filter_by(slug="births").first()
-        )
+        updatedDash = db.session.query(Dashboard).filter_by(slug="births").first()
         self.assertIn("color_namespace", updatedDash.json_metadata)
         self.assertIn("color_scheme", updatedDash.json_metadata)
         self.assertIn("label_colors", updatedDash.json_metadata)
@@ -203,7 +191,7 @@ class DashboardTests(SupersetTestCase):
 
     def test_copy_dash(self, username="admin"):
         self.login(username=username)
-        dash = db.session.query(models.Dashboard).filter_by(slug="births").first()
+        dash = db.session.query(Dashboard).filter_by(slug="births").first()
         positions = self.get_mock_positions(dash)
         new_label_colors = {"data value": "random color"}
         data = {
@@ -221,7 +209,7 @@ class DashboardTests(SupersetTestCase):
         dash_id = dash.id
         url = "/superset/save_dash/{}/".format(dash_id)
         self.client.post(url, data=dict(data=json.dumps(data)))
-        dash = db.session.query(models.Dashboard).filter_by(id=dash_id).first()
+        dash = db.session.query(Dashboard).filter_by(id=dash_id).first()
         orig_json_data = dash.data
 
         # Verify that copy matches original
@@ -234,19 +222,17 @@ class DashboardTests(SupersetTestCase):
         # exclude modified and changed_on attribute
         for index, slc in enumerate(orig_json_data["slices"]):
             for key in slc:
-                if key not in ["modified", "changed_on"]:
+                if key not in ["modified", "changed_on", "changed_on_humanized"]:
                     self.assertEqual(slc[key], resp["slices"][index][key])
 
     def test_add_slices(self, username="admin"):
         self.login(username=username)
-        dash = db.session.query(models.Dashboard).filter_by(slug="births").first()
+        dash = db.session.query(Dashboard).filter_by(slug="births").first()
         new_slice = (
-            db.session.query(models.Slice)
-            .filter_by(slice_name="Energy Force Layout")
-            .first()
+            db.session.query(Slice).filter_by(slice_name="Energy Force Layout").first()
         )
         existing_slice = (
-            db.session.query(models.Slice).filter_by(slice_name="Name Cloud").first()
+            db.session.query(Slice).filter_by(slice_name="Girl Name Cloud").first()
         )
         data = {
             "slice_ids": [new_slice.data["slice_id"], existing_slice.data["slice_id"]]
@@ -255,23 +241,21 @@ class DashboardTests(SupersetTestCase):
         resp = self.client.post(url, data=dict(data=json.dumps(data)))
         assert "SLICES ADDED" in resp.data.decode("utf-8")
 
-        dash = db.session.query(models.Dashboard).filter_by(slug="births").first()
+        dash = db.session.query(Dashboard).filter_by(slug="births").first()
         new_slice = (
-            db.session.query(models.Slice)
-            .filter_by(slice_name="Energy Force Layout")
-            .first()
+            db.session.query(Slice).filter_by(slice_name="Energy Force Layout").first()
         )
         assert new_slice in dash.slices
         assert len(set(dash.slices)) == len(dash.slices)
 
         # cleaning up
-        dash = db.session.query(models.Dashboard).filter_by(slug="births").first()
+        dash = db.session.query(Dashboard).filter_by(slug="births").first()
         dash.slices = [o for o in dash.slices if o.slice_name != "Energy Force Layout"]
         db.session.commit()
 
     def test_remove_slices(self, username="admin"):
         self.login(username=username)
-        dash = db.session.query(models.Dashboard).filter_by(slug="births").first()
+        dash = db.session.query(Dashboard).filter_by(slug="births").first()
         origin_slices_length = len(dash.slices)
 
         positions = self.get_mock_positions(dash)
@@ -293,7 +277,7 @@ class DashboardTests(SupersetTestCase):
         dash_id = dash.id
         url = "/superset/save_dash/{}/".format(dash_id)
         self.client.post(url, data=dict(data=json.dumps(data)))
-        dash = db.session.query(models.Dashboard).filter_by(id=dash_id).first()
+        dash = db.session.query(Dashboard).filter_by(id=dash_id).first()
 
         # verify slices data
         data = dash.data
@@ -303,7 +287,7 @@ class DashboardTests(SupersetTestCase):
         table = db.session.query(SqlaTable).filter_by(table_name="birth_names").one()
 
         # Make the births dash published so it can be seen
-        births_dash = db.session.query(models.Dashboard).filter_by(slug="births").one()
+        births_dash = db.session.query(Dashboard).filter_by(slug="births").one()
         births_dash.published = True
 
         db.session.merge(births_dash)
@@ -313,27 +297,27 @@ class DashboardTests(SupersetTestCase):
         self.revoke_public_access_to_table(table)
         self.logout()
 
-        resp = self.get_resp("/chart/list/")
-        self.assertNotIn("birth_names</a>", resp)
+        resp = self.get_resp("/api/v1/chart/")
+        self.assertNotIn("birth_names", resp)
 
-        resp = self.get_resp("/dashboard/list/")
+        resp = self.get_resp("/api/v1/dashboard/")
         self.assertNotIn("/superset/dashboard/births/", resp)
 
         self.grant_public_access_to_table(table)
 
         # Try access after adding appropriate permissions.
-        self.assertIn("birth_names", self.get_resp("/chart/list/"))
+        self.assertIn("birth_names", self.get_resp("/api/v1/chart/"))
 
-        resp = self.get_resp("/dashboard/list/")
+        resp = self.get_resp("/api/v1/dashboard/")
         self.assertIn("/superset/dashboard/births/", resp)
 
         self.assertIn("Births", self.get_resp("/superset/dashboard/births/"))
 
         # Confirm that public doesn't have access to other datasets.
-        resp = self.get_resp("/chart/list/")
-        self.assertNotIn("wb_health_population</a>", resp)
+        resp = self.get_resp("/api/v1/chart/")
+        self.assertNotIn("wb_health_population", resp)
 
-        resp = self.get_resp("/dashboard/list/")
+        resp = self.get_resp("/api/v1/dashboard/")
         self.assertNotIn("/superset/dashboard/world_health/", resp)
 
     def test_dashboard_with_created_by_can_be_accessed_by_public_users(self):
@@ -341,7 +325,7 @@ class DashboardTests(SupersetTestCase):
         table = db.session.query(SqlaTable).filter_by(table_name="birth_names").one()
         self.grant_public_access_to_table(table)
 
-        dash = db.session.query(models.Dashboard).filter_by(slug="births").first()
+        dash = db.session.query(Dashboard).filter_by(slug="births").first()
         dash.owners = [security_manager.find_user("admin")]
         dash.created_by = security_manager.find_user("admin")
         db.session.merge(dash)
@@ -350,7 +334,7 @@ class DashboardTests(SupersetTestCase):
         assert "Births" in self.get_resp("/superset/dashboard/births/")
 
     def test_only_owners_can_save(self):
-        dash = db.session.query(models.Dashboard).filter_by(slug="births").first()
+        dash = db.session.query(Dashboard).filter_by(slug="births").first()
         dash.owners = []
         db.session.merge(dash)
         db.session.commit()
@@ -361,18 +345,16 @@ class DashboardTests(SupersetTestCase):
 
         alpha = security_manager.find_user("alpha")
 
-        dash = db.session.query(models.Dashboard).filter_by(slug="births").first()
+        dash = db.session.query(Dashboard).filter_by(slug="births").first()
         dash.owners = [alpha]
         db.session.merge(dash)
         db.session.commit()
         self.test_save_dash("alpha")
 
     def test_owners_can_view_empty_dashboard(self):
-        dash = (
-            db.session.query(models.Dashboard).filter_by(slug="empty_dashboard").first()
-        )
+        dash = db.session.query(Dashboard).filter_by(slug="empty_dashboard").first()
         if not dash:
-            dash = models.Dashboard()
+            dash = Dashboard()
             dash.dashboard_title = "Empty Dashboard"
             dash.slug = "empty_dashboard"
         else:
@@ -384,28 +366,29 @@ class DashboardTests(SupersetTestCase):
         gamma_user = security_manager.find_user("gamma")
         self.login(gamma_user.username)
 
-        resp = self.get_resp("/dashboard/list/")
+        resp = self.get_resp("/api/v1/dashboard/")
         self.assertNotIn("/superset/dashboard/empty_dashboard/", resp)
 
     def test_users_can_view_published_dashboard(self):
         table = db.session.query(SqlaTable).filter_by(table_name="energy_usage").one()
         # get a slice from the allowed table
-        slice = (
-            db.session.query(models.Slice).filter_by(slice_name="Energy Sankey").one()
-        )
+        slice = db.session.query(Slice).filter_by(slice_name="Energy Sankey").one()
 
         self.grant_public_access_to_table(table)
 
+        hidden_dash_slug = f"hidden_dash_{random()}"
+        published_dash_slug = f"published_dash_{random()}"
+
         # Create a published and hidden dashboard and add them to the database
-        published_dash = models.Dashboard()
+        published_dash = Dashboard()
         published_dash.dashboard_title = "Published Dashboard"
-        published_dash.slug = "published_dash"
+        published_dash.slug = published_dash_slug
         published_dash.slices = [slice]
         published_dash.published = True
 
-        hidden_dash = models.Dashboard()
+        hidden_dash = Dashboard()
         hidden_dash.dashboard_title = "Hidden Dashboard"
-        hidden_dash.slug = "hidden_dash"
+        hidden_dash.slug = hidden_dash_slug
         hidden_dash.slices = [slice]
         hidden_dash.published = False
 
@@ -413,23 +396,25 @@ class DashboardTests(SupersetTestCase):
         db.session.merge(hidden_dash)
         db.session.commit()
 
-        resp = self.get_resp("/dashboard/list/")
-        self.assertNotIn("/superset/dashboard/hidden_dash/", resp)
-        self.assertIn("/superset/dashboard/published_dash/", resp)
+        resp = self.get_resp("/api/v1/dashboard/")
+        self.assertNotIn(f"/superset/dashboard/{hidden_dash_slug}/", resp)
+        self.assertIn(f"/superset/dashboard/{published_dash_slug}/", resp)
 
     def test_users_can_view_own_dashboard(self):
         user = security_manager.find_user("gamma")
+        my_dash_slug = f"my_dash_{random()}"
+        not_my_dash_slug = f"not_my_dash_{random()}"
 
         # Create one dashboard I own and another that I don't
-        dash = models.Dashboard()
+        dash = Dashboard()
         dash.dashboard_title = "My Dashboard"
-        dash.slug = "my_dash"
+        dash.slug = my_dash_slug
         dash.owners = [user]
         dash.slices = []
 
-        hidden_dash = models.Dashboard()
+        hidden_dash = Dashboard()
         hidden_dash.dashboard_title = "Not My Dashboard"
-        hidden_dash.slug = "not_my_dash"
+        hidden_dash.slug = not_my_dash_slug
         hidden_dash.slices = []
         hidden_dash.owners = []
 
@@ -439,30 +424,28 @@ class DashboardTests(SupersetTestCase):
 
         self.login(user.username)
 
-        resp = self.get_resp("/dashboard/list/")
-        self.assertIn("/superset/dashboard/my_dash/", resp)
-        self.assertNotIn("/superset/dashboard/not_my_dash/", resp)
+        resp = self.get_resp("/api/v1/dashboard/")
+        self.assertIn(f"/superset/dashboard/{my_dash_slug}/", resp)
+        self.assertNotIn(f"/superset/dashboard/{not_my_dash_slug}/", resp)
 
     def test_users_can_view_favorited_dashboards(self):
         user = security_manager.find_user("gamma")
+        fav_dash_slug = f"my_favorite_dash_{random()}"
+        regular_dash_slug = f"regular_dash_{random()}"
 
-        favorite_dash = models.Dashboard()
+        favorite_dash = Dashboard()
         favorite_dash.dashboard_title = "My Favorite Dashboard"
-        favorite_dash.slug = "my_favorite_dash"
+        favorite_dash.slug = fav_dash_slug
 
-        regular_dash = models.Dashboard()
+        regular_dash = Dashboard()
         regular_dash.dashboard_title = "A Plain Ol Dashboard"
-        regular_dash.slug = "regular_dash"
+        regular_dash.slug = regular_dash_slug
 
         db.session.merge(favorite_dash)
         db.session.merge(regular_dash)
         db.session.commit()
 
-        dash = (
-            db.session.query(models.Dashboard)
-            .filter_by(slug="my_favorite_dash")
-            .first()
-        )
+        dash = db.session.query(Dashboard).filter_by(slug=fav_dash_slug).first()
 
         favorites = models.FavStar()
         favorites.obj_id = dash.id
@@ -474,16 +457,16 @@ class DashboardTests(SupersetTestCase):
 
         self.login(user.username)
 
-        resp = self.get_resp("/dashboard/list/")
-        self.assertIn("/superset/dashboard/my_favorite_dash/", resp)
+        resp = self.get_resp("/api/v1/dashboard/")
+        self.assertIn(f"/superset/dashboard/{fav_dash_slug}/", resp)
 
     def test_user_can_not_view_unpublished_dash(self):
         admin_user = security_manager.find_user("admin")
         gamma_user = security_manager.find_user("gamma")
-        slug = "admin_owned_unpublished_dash"
+        slug = f"admin_owned_unpublished_dash_{random()}"
 
         # Create a dashboard owned by admin and unpublished
-        dash = models.Dashboard()
+        dash = Dashboard()
         dash.dashboard_title = "My Dashboard"
         dash.slug = slug
         dash.owners = [admin_user]
@@ -494,7 +477,7 @@ class DashboardTests(SupersetTestCase):
 
         # list dashboards as a gamma user
         self.login(gamma_user.username)
-        resp = self.get_resp("/dashboard/list/")
+        resp = self.get_resp("/api/v1/dashboard/")
         self.assertNotIn(f"/superset/dashboard/{slug}/", resp)
 
 
