@@ -14,15 +14,18 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
+# isort:skip_file
 from datetime import datetime, timedelta
-import unittest
 from unittest.mock import Mock, patch, PropertyMock
 
 from flask_babel import gettext as __
+import pytest
 from selenium.common.exceptions import WebDriverException
+from slack import errors, WebClient
 
-from superset import app, db
-from superset.models.core import Dashboard, Slice
+from tests.test_app import app
+from superset import db
+from superset.models.dashboard import Dashboard
 from superset.models.schedules import (
     DashboardEmailSchedule,
     EmailDeliveryType,
@@ -35,10 +38,13 @@ from superset.tasks.schedules import (
     deliver_slice,
     next_schedules,
 )
+from superset.models.slice import Slice
+from tests.base_tests import SupersetTestCase
+
 from .utils import read_fixture
 
 
-class SchedulesTestCase(unittest.TestCase):
+class TestSchedules(SupersetTestCase):
 
     RECIPIENTS = "recipient1@superset.com, recipient2@superset.com"
     BCC = "bcc@superset.com"
@@ -46,41 +52,46 @@ class SchedulesTestCase(unittest.TestCase):
 
     @classmethod
     def setUpClass(cls):
-        cls.common_data = dict(
-            active=True,
-            crontab="* * * * *",
-            recipients=cls.RECIPIENTS,
-            deliver_as_group=True,
-            delivery_type=EmailDeliveryType.inline,
-        )
+        with app.app_context():
+            cls.common_data = dict(
+                active=True,
+                crontab="* * * * *",
+                recipients=cls.RECIPIENTS,
+                deliver_as_group=True,
+                delivery_type=EmailDeliveryType.inline,
+            )
 
-        # Pick up a random slice and dashboard
-        slce = db.session.query(Slice).all()[0]
-        dashboard = db.session.query(Dashboard).all()[0]
+            # Pick up a random slice and dashboard
+            slce = db.session.query(Slice).all()[0]
+            dashboard = db.session.query(Dashboard).all()[0]
 
-        dashboard_schedule = DashboardEmailSchedule(**cls.common_data)
-        dashboard_schedule.dashboard_id = dashboard.id
-        dashboard_schedule.user_id = 1
-        db.session.add(dashboard_schedule)
+            dashboard_schedule = DashboardEmailSchedule(**cls.common_data)
+            dashboard_schedule.dashboard_id = dashboard.id
+            dashboard_schedule.user_id = 1
+            db.session.add(dashboard_schedule)
 
-        slice_schedule = SliceEmailSchedule(**cls.common_data)
-        slice_schedule.slice_id = slce.id
-        slice_schedule.user_id = 1
-        slice_schedule.email_format = SliceEmailReportFormat.data
+            slice_schedule = SliceEmailSchedule(**cls.common_data)
+            slice_schedule.slice_id = slce.id
+            slice_schedule.user_id = 1
+            slice_schedule.email_format = SliceEmailReportFormat.data
+            slice_schedule.slack_channel = "#test_channel"
 
-        db.session.add(slice_schedule)
-        db.session.commit()
+            db.session.add(slice_schedule)
+            db.session.commit()
 
-        cls.slice_schedule = slice_schedule.id
-        cls.dashboard_schedule = dashboard_schedule.id
+            cls.slice_schedule = slice_schedule.id
+            cls.dashboard_schedule = dashboard_schedule.id
 
     @classmethod
     def tearDownClass(cls):
-        db.session.query(SliceEmailSchedule).filter_by(id=cls.slice_schedule).delete()
-        db.session.query(DashboardEmailSchedule).filter_by(
-            id=cls.dashboard_schedule
-        ).delete()
-        db.session.commit()
+        with app.app_context():
+            db.session.query(SliceEmailSchedule).filter_by(
+                id=cls.slice_schedule
+            ).delete()
+            db.session.query(DashboardEmailSchedule).filter_by(
+                id=cls.dashboard_schedule
+            ).delete()
+            db.session.commit()
 
     def test_crontab_scheduler(self):
         crontab = "* * * * *"
@@ -182,7 +193,14 @@ class SchedulesTestCase(unittest.TestCase):
             .all()[0]
         )
 
-        deliver_dashboard(schedule)
+        deliver_dashboard(
+            schedule.dashboard_id,
+            schedule.recipients,
+            schedule.slack_channel,
+            schedule.delivery_type,
+            schedule.deliver_as_group,
+        )
+
         mtime.sleep.assert_called_once()
         driver.screenshot.assert_not_called()
         send_email_smtp.assert_called_once()
@@ -212,14 +230,21 @@ class SchedulesTestCase(unittest.TestCase):
         )
 
         schedule.delivery_type = EmailDeliveryType.attachment
-        deliver_dashboard(schedule)
+
+        deliver_dashboard(
+            schedule.dashboard_id,
+            schedule.recipients,
+            schedule.slack_channel,
+            schedule.delivery_type,
+            schedule.deliver_as_group,
+        )
 
         mtime.sleep.assert_called_once()
         driver.screenshot.assert_not_called()
         send_email_smtp.assert_called_once()
         self.assertIsNone(send_email_smtp.call_args[1]["images"])
-        self.assertEquals(
-            send_email_smtp.call_args[1]["data"]["screenshot.png"],
+        self.assertEqual(
+            send_email_smtp.call_args[1]["data"]["screenshot"],
             element.screenshot_as_png,
         )
 
@@ -248,13 +273,20 @@ class SchedulesTestCase(unittest.TestCase):
             .all()[0]
         )
 
-        deliver_dashboard(schedule)
+        deliver_dashboard(
+            schedule.dashboard_id,
+            schedule.recipients,
+            schedule.slack_channel,
+            schedule.delivery_type,
+            schedule.deliver_as_group,
+        )
+
         mtime.sleep.assert_called_once()
         driver.screenshot.assert_called_once()
         send_email_smtp.assert_called_once()
 
-        self.assertEquals(send_email_smtp.call_args[0][0], self.RECIPIENTS)
-        self.assertEquals(
+        self.assertEqual(send_email_smtp.call_args[0][0], self.RECIPIENTS)
+        self.assertEqual(
             list(send_email_smtp.call_args[1]["images"].values())[0],
             driver.screenshot.return_value,
         )
@@ -286,17 +318,27 @@ class SchedulesTestCase(unittest.TestCase):
         # Set a bcc email address
         app.config["EMAIL_REPORT_BCC_ADDRESS"] = self.BCC
 
-        deliver_dashboard(schedule)
+        deliver_dashboard(
+            schedule.dashboard_id,
+            schedule.recipients,
+            schedule.slack_channel,
+            schedule.delivery_type,
+            schedule.deliver_as_group,
+        )
+
         mtime.sleep.assert_called_once()
         driver.screenshot.assert_not_called()
 
-        self.assertEquals(send_email_smtp.call_count, 2)
-        self.assertEquals(send_email_smtp.call_args[1]["bcc"], self.BCC)
+        self.assertEqual(send_email_smtp.call_count, 2)
+        self.assertEqual(send_email_smtp.call_args[1]["bcc"], self.BCC)
 
+    @patch("superset.tasks.slack_util.WebClient.files_upload")
     @patch("superset.tasks.schedules.firefox.webdriver.WebDriver")
     @patch("superset.tasks.schedules.send_email_smtp")
     @patch("superset.tasks.schedules.time")
-    def test_deliver_slice_inline_image(self, mtime, send_email_smtp, driver_class):
+    def test_deliver_slice_inline_image(
+        self, mtime, send_email_smtp, driver_class, files_upload
+    ):
         element = Mock()
         driver = Mock()
         mtime.sleep.return_value = None
@@ -317,20 +359,40 @@ class SchedulesTestCase(unittest.TestCase):
         schedule.email_format = SliceEmailReportFormat.visualization
         schedule.delivery_format = EmailDeliveryType.inline
 
-        deliver_slice(schedule)
+        deliver_slice(
+            schedule.slice_id,
+            schedule.recipients,
+            schedule.slack_channel,
+            schedule.delivery_type,
+            schedule.email_format,
+            schedule.deliver_as_group,
+        )
         mtime.sleep.assert_called_once()
         driver.screenshot.assert_not_called()
         send_email_smtp.assert_called_once()
 
-        self.assertEquals(
+        self.assertEqual(
             list(send_email_smtp.call_args[1]["images"].values())[0],
             element.screenshot_as_png,
         )
 
+        self.assertEqual(
+            files_upload.call_args[1],
+            {
+                "channels": "#test_channel",
+                "file": element.screenshot_as_png,
+                "initial_comment": f"\n        *Participants*\n\n        <http://0.0.0.0:8080/superset/slice/{schedule.slice_id}/|Explore in Superset>\n        ",
+                "title": "[Report]  Participants",
+            },
+        )
+
+    @patch("superset.tasks.slack_util.WebClient.files_upload")
     @patch("superset.tasks.schedules.firefox.webdriver.WebDriver")
     @patch("superset.tasks.schedules.send_email_smtp")
     @patch("superset.tasks.schedules.time")
-    def test_deliver_slice_attachment(self, mtime, send_email_smtp, driver_class):
+    def test_deliver_slice_attachment(
+        self, mtime, send_email_smtp, driver_class, files_upload
+    ):
         element = Mock()
         driver = Mock()
         mtime.sleep.return_value = None
@@ -351,27 +413,46 @@ class SchedulesTestCase(unittest.TestCase):
         schedule.email_format = SliceEmailReportFormat.visualization
         schedule.delivery_type = EmailDeliveryType.attachment
 
-        deliver_slice(schedule)
+        deliver_slice(
+            schedule.slice_id,
+            schedule.recipients,
+            schedule.slack_channel,
+            schedule.delivery_type,
+            schedule.email_format,
+            schedule.deliver_as_group,
+        )
+
         mtime.sleep.assert_called_once()
         driver.screenshot.assert_not_called()
         send_email_smtp.assert_called_once()
 
-        self.assertEquals(
-            send_email_smtp.call_args[1]["data"]["screenshot.png"],
+        self.assertEqual(
+            send_email_smtp.call_args[1]["data"]["screenshot"],
             element.screenshot_as_png,
         )
 
+        self.assertEqual(
+            files_upload.call_args[1],
+            {
+                "channels": "#test_channel",
+                "file": element.screenshot_as_png,
+                "initial_comment": f"\n        *Participants*\n\n        <http://0.0.0.0:8080/superset/slice/{schedule.slice_id}/|Explore in Superset>\n        ",
+                "title": "[Report]  Participants",
+            },
+        )
+
+    @patch("superset.tasks.slack_util.WebClient.files_upload")
     @patch("superset.tasks.schedules.urllib.request.OpenerDirector.open")
     @patch("superset.tasks.schedules.urllib.request.urlopen")
     @patch("superset.tasks.schedules.send_email_smtp")
     def test_deliver_slice_csv_attachment(
-        self, send_email_smtp, mock_open, mock_urlopen
+        self, send_email_smtp, mock_open, mock_urlopen, files_upload
     ):
         response = Mock()
         mock_open.return_value = response
         mock_urlopen.return_value = response
         mock_urlopen.return_value.getcode.return_value = 200
-        response.content = self.CSV
+        response.read.return_value = self.CSV
 
         schedule = (
             db.session.query(SliceEmailSchedule)
@@ -382,23 +463,43 @@ class SchedulesTestCase(unittest.TestCase):
         schedule.email_format = SliceEmailReportFormat.data
         schedule.delivery_type = EmailDeliveryType.attachment
 
-        deliver_slice(schedule)
+        deliver_slice(
+            schedule.slice_id,
+            schedule.recipients,
+            schedule.slack_channel,
+            schedule.delivery_type,
+            schedule.email_format,
+            schedule.deliver_as_group,
+        )
+
         send_email_smtp.assert_called_once()
 
         file_name = __("%(name)s.csv", name=schedule.slice.slice_name)
 
-        self.assertEquals(send_email_smtp.call_args[1]["data"][file_name], self.CSV)
+        self.assertEqual(send_email_smtp.call_args[1]["data"][file_name], self.CSV)
 
+        self.assertEqual(
+            files_upload.call_args[1],
+            {
+                "channels": "#test_channel",
+                "file": self.CSV,
+                "initial_comment": f"\n        *Participants*\n\n        <http://0.0.0.0:8080/superset/slice/{schedule.slice_id}/|Explore in Superset>\n        ",
+                "title": "[Report]  Participants",
+            },
+        )
+
+    @patch("superset.tasks.slack_util.WebClient.files_upload")
     @patch("superset.tasks.schedules.urllib.request.urlopen")
     @patch("superset.tasks.schedules.urllib.request.OpenerDirector.open")
     @patch("superset.tasks.schedules.send_email_smtp")
-    def test_deliver_slice_csv_inline(self, send_email_smtp, mock_open, mock_urlopen):
+    def test_deliver_slice_csv_inline(
+        self, send_email_smtp, mock_open, mock_urlopen, files_upload
+    ):
         response = Mock()
         mock_open.return_value = response
         mock_urlopen.return_value = response
         mock_urlopen.return_value.getcode.return_value = 200
-        response.content = self.CSV
-
+        response.read.return_value = self.CSV
         schedule = (
             db.session.query(SliceEmailSchedule)
             .filter_by(id=self.slice_schedule)
@@ -408,8 +509,34 @@ class SchedulesTestCase(unittest.TestCase):
         schedule.email_format = SliceEmailReportFormat.data
         schedule.delivery_type = EmailDeliveryType.inline
 
-        deliver_slice(schedule)
+        deliver_slice(
+            schedule.slice_id,
+            schedule.recipients,
+            schedule.slack_channel,
+            schedule.delivery_type,
+            schedule.email_format,
+            schedule.deliver_as_group,
+        )
+
         send_email_smtp.assert_called_once()
 
         self.assertIsNone(send_email_smtp.call_args[1]["data"])
         self.assertTrue("<table " in send_email_smtp.call_args[0][2])
+
+        self.assertEqual(
+            files_upload.call_args[1],
+            {
+                "channels": "#test_channel",
+                "file": self.CSV,
+                "initial_comment": f"\n        *Participants*\n\n        <http://0.0.0.0:8080/superset/slice/{schedule.slice_id}/|Explore in Superset>\n        ",
+                "title": "[Report]  Participants",
+            },
+        )
+
+
+def test_slack_client_compatibility():
+    c2 = WebClient()
+    # slackclient >2.5.0 raises TypeError: a bytes-like object is required, not 'str
+    # and requires to path a filepath instead of the bytes directly
+    with pytest.raises(errors.SlackApiError):
+        c2.files_upload(channels="#bogdan-test2", file=b"blabla", title="Test upload")
